@@ -6,10 +6,11 @@
 import { BaseStore } from "../common/base-store";
 import * as path from "path";
 import type { LensExtension } from "./lens-extension";
+import assert from "assert";
 
 export abstract class ExtensionStore<T> extends BaseStore<T> {
   readonly displayName = "ExtensionStore<T>";
-  protected extension: LensExtension;
+  protected extension?: LensExtension;
 
   loadExtension(extension: LensExtension) {
     this.extension = extension;
@@ -24,6 +25,8 @@ export abstract class ExtensionStore<T> extends BaseStore<T> {
   }
 
   protected cwd() {
+    assert(this.extension, "must call this.load() first");
+
     return path.join(super.cwd(), "extension-store", this.extension.name);
   }
 }
