@@ -3,8 +3,7 @@
  * Licensed under MIT License. See LICENSE in root directory for more information.
  */
 
-import type { PrometheusProvider, PrometheusService } from "../prometheus/provider-registry";
-import { PrometheusProviderRegistry } from "../prometheus/provider-registry";
+import type { PrometheusProvider, PrometheusService, PrometheusProviderRegistry } from "../prometheus/provider-registry";
 import type { ClusterPrometheusPreferences } from "../../common/cluster-types";
 import type { Cluster } from "../../common/cluster/cluster";
 import type httpProxy from "http-proxy";
@@ -28,12 +27,25 @@ interface PrometheusServicePreferences {
 }
 
 interface Dependencies {
-  createKubeAuthProxy: CreateKubeAuthProxy;
-  authProxyCa: string;
+  readonly createKubeAuthProxy: CreateKubeAuthProxy;
+  readonly authProxyCa: string;
+  readonly prometheusProviderRegistry: PrometheusProviderRegistry;
 }
 
-export class ContextHandler {
-  public clusterUrl: UrlWithStringQuery;
+export interface ClusterContextHandler {
+  readonly clusterUrl: UrlWithStringQuery;
+  setupPrometheus(preferences?: ClusterPrometheusPreferences): void;
+  getPrometheusDetails(): Promise<PrometheusDetails>;
+  resolveAuthProxyUrl(): Promise<string>;
+  resolveAuthProxyCa(): string;
+  getApiTarget(isLongRunningRequest?: boolean): Promise<httpProxy.ServerOptions>;
+  restartServer(): Promise<void>;
+  ensureServer(): Promise<void>;
+  stopServer(): void;
+}
+
+export class ContextHandler implements ClusterContextHandler {
+  public readonly clusterUrl: UrlWithStringQuery;
   protected kubeAuthProxy?: KubeAuthProxy;
   protected apiTarget?: httpProxy.ServerOptions;
   protected prometheusProvider?: string;
@@ -46,7 +58,7 @@ export class ContextHandler {
 
   public setupPrometheus(preferences: ClusterPrometheusPreferences = {}) {
     this.prometheusProvider = preferences.prometheusProvider?.type;
-    this.prometheus = preferences.prometheus || null;
+    this.prometheus = preferences.prometheus;
   }
 
   public async getPrometheusDetails(): Promise<PrometheusDetails> {
@@ -67,11 +79,11 @@ export class ContextHandler {
       this.prometheusProvider = service.id;
     }
 
-    return PrometheusProviderRegistry.getInstance().getByKind(this.prometheusProvider);
+    return this.dependencies.prometheusProviderRegistry.getByKind(this.prometheusProvider);
   }
 
   protected listPotentialProviders(): PrometheusProvider[] {
-    const registry = PrometheusProviderRegistry.getInstance();
+    const registry = this.dependencies.prometheusProviderRegistry;
     const provider = this.prometheusProvider && registry.getByKind(this.prometheusProvider);
 
     if (provider) {
@@ -82,7 +94,7 @@ export class ContextHandler {
   }
 
   protected async getPrometheusService(): Promise<PrometheusService> {
-    if (this.prometheus !== null && this.prometheusProvider !== null) {
+    if (this.prometheus && this.prometheusProvider) {
       return {
         id: this.prometheusProvider,
         namespace: this.prometheus.namespace,
@@ -117,11 +129,10 @@ export class ContextHandler {
     throw Object.assign(new Error("No Prometheus service found"), { cause: errors });
   }
 
-  async resolveAuthProxyUrl() {
-    await this.ensureServer();
-    const path = this.clusterUrl.path !== "/" ? this.clusterUrl.path : "";
+  async resolveAuthProxyUrl(): Promise<string> {
+    const kubeAuthProxy = await this.ensureServerHelper();
 
-    return `https://127.0.0.1:${this.kubeAuthProxy.port}${this.kubeAuthProxy.apiPrefix}${path}`;
+    return `https://127.0.0.1:${kubeAuthProxy.port}${kubeAuthProxy.apiPrefix}`;
   }
 
   resolveAuthProxyCa() {
@@ -139,26 +150,29 @@ export class ContextHandler {
   }
 
   protected async newApiTarget(timeout: number): Promise<httpProxy.ServerOptions> {
-    await this.ensureServer();
+    const kubeAuthProxy = await this.ensureServerHelper();
+    const headers: Record<string, string> = {};
+
+    if (this.clusterUrl.hostname) {
+      headers.Host = this.clusterUrl.hostname;
+    }
 
     return {
       target: {
         protocol: "https:",
         host: "127.0.0.1",
-        port: this.kubeAuthProxy.port,
-        path: this.kubeAuthProxy.apiPrefix,
-        ca: this.dependencies.authProxyCa,
+        port: kubeAuthProxy.port,
+        path: kubeAuthProxy.apiPrefix,
+        ca: this.resolveAuthProxyCa(),
       },
       changeOrigin: true,
       timeout,
       secure: true,
-      headers: {
-        "Host": this.clusterUrl.hostname,
-      },
+      headers,
     };
   }
 
-  async ensureServer() {
+  protected async ensureServerHelper(): Promise<KubeAuthProxy> {
     if (!this.kubeAuthProxy) {
       const proxyEnv = Object.assign({}, process.env);
 
@@ -167,9 +181,23 @@ export class ContextHandler {
       }
       this.kubeAuthProxy = this.dependencies.createKubeAuthProxy(this.cluster, proxyEnv);
       await this.kubeAuthProxy.run();
+
+      return this.kubeAuthProxy;
     }
 
     await this.kubeAuthProxy.whenReady;
+
+    return this.kubeAuthProxy;
+  }
+
+  async ensureServer(): Promise<void> {
+    await this.ensureServerHelper();
+  }
+
+  async restartServer(): Promise<void> {
+    this.stopServer();
+
+    await this.ensureServerHelper();
   }
 
   stopServer() {
