@@ -2,24 +2,19 @@
  * Copyright (c) OpenLens Authors. All rights reserved.
  * Licensed under MIT License. See LICENSE in root directory for more information.
  */
-import { app, dialog, Menu } from "electron";
-import type { IComputedValue } from "mobx";
-import { autorun } from "mobx";
+import { app, Menu } from "electron";
 import { appName, isWindows, productName } from "../../common/vars";
 import packageJson from "../../../package.json";
 import type { MenuItemOpts } from "./application-menu-items.injectable";
+import type { ShowMessagePopup } from "../electron-app/features/show-message-popup.injectable";
 
 export type MenuTopId = "mac" | "file" | "edit" | "view" | "help";
 
-export function initMenu(
-  applicationMenuItems: IComputedValue<MenuItemOpts[]>,
-) {
-  return autorun(() => buildMenu(applicationMenuItems.get()), {
-    delay: 100,
-  });
+interface Dependencies {
+  showMessagePopup: ShowMessagePopup;
 }
 
-export function showAbout() {
+export const showAbout = ({ showMessagePopup }: Dependencies) => async () => {
   const appInfo = [
     `${appName}: ${app.getVersion()}`,
     `Electron: ${process.versions.electron}`,
@@ -28,18 +23,14 @@ export function showAbout() {
     packageJson.copyright,
   ];
 
-  dialog.showMessageBoxSync({
-    title: `${isWindows ? " ".repeat(2) : ""}${appName}`,
-    type: "info",
-    buttons: ["Close"],
-    message: productName,
-    detail: appInfo.join("\r\n"),
-  });
-}
+  await showMessagePopup(
+    `${isWindows ? " ".repeat(2) : ""}${appName}`,
+    productName,
+    appInfo.join("\r\n"),
+  );
+};
 
-export function buildMenu(
-  applicationMenuItems: MenuItemOpts[],
-) {
+export function buildMenu(applicationMenuItems: MenuItemOpts[]) {
   Menu.setApplicationMenu(
     Menu.buildFromTemplate(applicationMenuItems),
   );
