@@ -6,6 +6,7 @@ import { getInjectable } from "@ogre-tools/injectable";
 import extensionLoaderInjectable from "../../../extensions/extension-loader/extension-loader.injectable";
 import { LensProtocolRouterMain } from "./lens-protocol-router-main";
 import extensionsStoreInjectable from "../../../extensions/extensions-store/extensions-store.injectable";
+import showApplicationWindowInjectable from "../../start-main-application/lens-window/show-application-window.injectable";
 
 const lensProtocolRouterMainInjectable = getInjectable({
   id: "lens-protocol-router-main",
@@ -14,6 +15,7 @@ const lensProtocolRouterMainInjectable = getInjectable({
     new LensProtocolRouterMain({
       extensionLoader: di.inject(extensionLoaderInjectable),
       extensionsStore: di.inject(extensionsStoreInjectable),
+      showApplicationWindow: di.inject(showApplicationWindowInjectable),
     }),
 });
 
