@@ -4,7 +4,7 @@
  */
 import { getInjectable } from "@ogre-tools/injectable";
 import { observable } from "mobx";
-import type { ConfirmDialogParams } from ".";
+import type { ConfirmDialogParams } from "./confirm-dialog";
 
 const confirmDialogStateInjectable = getInjectable({
   id: "confirm-dialog-state",
