@@ -4,15 +4,14 @@
  */
 
 import React from "react";
-import type { IPodMetrics } from "../../../common/k8s-api/endpoints";
-import type { IMetrics } from "../../../common/k8s-api/endpoints/metrics.api";
+import type { PodMetricData } from "../../../common/k8s-api/endpoints";
 import { getMetricLastPoints } from "../../../common/k8s-api/endpoints/metrics.api";
 import { bytesToUnits } from "../../utils";
 import { Badge } from "../badge";
 import { DrawerItem } from "../drawer";
 
 export interface ResourceMetricsTextProps {
-  metrics: IPodMetrics<IMetrics> | null | undefined;
+  metrics: PodMetricData | null | undefined;
 }
 
 export function ResourceMetricsText({ metrics }: ResourceMetricsTextProps) {
