@@ -7,6 +7,7 @@ import type { ClusterDependencies } from "../../common/cluster/cluster";
 import { Cluster } from "../../common/cluster/cluster";
 import directoryForKubeConfigsInjectable from "../../common/app-paths/directory-for-kube-configs/directory-for-kube-configs.injectable";
 import { createClusterInjectionToken } from "../../common/cluster/create-cluster-injection-token";
+import loggerInjectable from "../../common/logger.injectable";
 
 const createClusterInjectable = getInjectable({
   id: "create-cluster",
@@ -14,14 +15,20 @@ const createClusterInjectable = getInjectable({
   instantiate: (di) => {
     const dependencies: ClusterDependencies = {
       directoryForKubeConfigs: di.inject(directoryForKubeConfigsInjectable),
-      createKubeconfigManager: () => { throw new Error("Tried to access back-end feature in front-end."); },
+      logger: di.inject(loggerInjectable),
+
+      // TODO: Dismantle wrong abstraction
+      // Note: "as never" to get around strictness in unnatural scenario
+      createKubeconfigManager: () => undefined as never,
       createKubectl: () => { throw new Error("Tried to access back-end feature in front-end.");},
-      createContextHandler: () => { throw new Error("Tried to access back-end feature in front-end."); },
+      createContextHandler: () => undefined as never,
       createAuthorizationReview: () => { throw new Error("Tried to access back-end feature in front-end."); },
       createListNamespaces: () => { throw new Error("Tried to access back-end feature in front-end."); },
+      detectorRegistry: undefined as never,
+      createVersionDetector: () => { throw new Error("Tried to access back-end feature in front-end."); },
     };
 
-    return (model) => new Cluster(dependencies, model);
+    return (model, configData) => new Cluster(dependencies, model, configData);
   },
 
   injectionToken: createClusterInjectionToken,
